@@ -13,6 +13,7 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
+from functools import wraps, partial
 import random
 
 import sys
@@ -32,8 +33,9 @@ from calculate.update.datavars import DataVarsUpdate
 from calculate.update.update_info import UpdateInfo
 from calculate.lib.cl_log import log
 import re
-from itertools import chain
+import shutil
 from collections import MutableSet
+from update_tasks import EmergeMark
 
 from calculate.lib.utils.portage import (Git, Layman, EmergeLog, GitError,
                                          EmergeLogNamedTask, PackageList,
@@ -48,7 +50,8 @@ from calculate.lib.utils.files import (getProgPath, STDOUT, removeDir,
                                        readFile, listDirectory)
 import emerge_parser
 import logging
-from emerge_parser import EmergeParser, EmergeCommand, EmergeError, EmergeCache
+from emerge_parser import (EmergeParser, EmergeCommand, EmergeError,
+                           EmergeCache, ChrootEmergeCommand)
 
 from calculate.lib.cl_lang import (setLocalTranslate, getLazyLocalTranslate,
                                    RegexpLocalization, _)
@@ -63,18 +66,19 @@ class OverlayOwnCache(MutableSet):
     """
     Сет оверлеев с интегрированным кэшем
     """
-    def __init__(self, initvalue=()):
-        pass
+    def __init__(self, dv=None):
+        self.dv = dv
 
     def __get_overlays(self):
-        own_cache_value = SystemIni().getVar('update', 'own_cache') or ""
+        own_cache_value = SystemIni(self.dv).getVar('update', 'own_cache') or ""
         return [x.strip() for x in own_cache_value.split(',') if x.strip()]
 
     def __write_overlays(self, overlays):
         if not overlays:
-            SystemIni().delVar('update', 'own_cache')
+            SystemIni(self.dv).delVar('update', 'own_cache')
         else:
-            SystemIni().setVar('update', {'own_cache': ",".join(overlays)})
+            SystemIni(self.dv).setVar('update',
+                                      {'own_cache': ",".join(overlays)})
 
     def __contains__(self, item):
         return item in self.__get_overlays()
@@ -101,6 +105,21 @@ class OverlayOwnCache(MutableSet):
             self.__write_overlays(overlays)
 
 
+def variable_module(var_env):
+    def variable_module_decor(f):
+        @wraps(f)
+        def wrapper(self, *args, **kw):
+            old_env = self.clVars.defaultModule
+            try:
+                self.clVars.defaultModule = var_env
+                return f(self, *args, **kw)
+            finally:
+                self.clVars.defaultModule = old_env
+        return wrapper
+
+    return variable_module_decor
+
+
 class Update(object):
     """Основной объект для выполнения действий связанных с обновлением системы
 
@@ -118,8 +137,12 @@ class Update(object):
             map(emerge_parser.GitCheckvalue,
                 self.clVars.Get('update.cl_update_rep_path')))
         self.update_map = {}
+        self.refresh_binhost = False
 
-    def _syncRepository(self, name, url, rpath, revision, branch,
+    def get_prog_path(self, progname):
+        return getProgPath(progname)
+
+    def _syncRepository(self, name, url, rpath, revision,
                         cb_progress=None):
         """
         Синхронизировать репозитори
@@ -130,43 +153,38 @@ class Update(object):
         try:
             self.stash_cache(rpath, name)
             if not git.checkExistsRep(rpath):
-                if revision == "last":
-                    git.cloneRepository(url, rpath, branch,
-                                        cb_progress=cb_progress)
-                else:
-                    git.cloneRevRepository(url, rpath, branch, revision,
-                                           cb_progress=cb_progress)
+                git.cloneTagRepository(url, rpath, revision,
+                                       cb_progress=cb_progress)
                 info_outdate = True
             else:
-                # если нужно обновиться до конкретной ревизии
-                if revision != "last":
-                    if revision == git.getCurrentCommit(rpath):
-                        if git.getBranch(rpath) == branch:
-                            return False
-                # получить изменения из удаленного репозитория
-                git.fetchRepository(rpath, cb_progress=cb_progress)
-                # если текущая ветка не соответствует нужной
-                repInfo = git.getStatusInfo(rpath)
-                if repInfo['branch'] != branch:
-                    # меняем ветку
-                    info_outdate = True
-                    git.checkoutBranch(rpath, branch)
-                if revision == "last":
-                    if git.resetRepository(rpath, to_origin=True):
-                        # если не удалось сбросить
-                        repInfo = git.getStatusInfo(rpath)
-                        if repInfo.get("files", False):
-                            raise GitError("Failed to reset git")
+                try:
+                    cr = ""
+                    need_update = False
+                    tag_cr = git.getCommit(rpath, revision)
+                    cr = git.getCurrentCommit(rpath)
+                    ref_type = git.reference_type(rpath, revision)
+                    status = git.getStatusInfo(rpath)
+                    if (tag_cr != cr or ref_type == Git.Reference.Branch or
+                        not status or status['files']):
+                        need_update = True
+                except GitError:
+                    need_update = True
+                if need_update:
+                    git.updateTagRepository(url, rpath, revision,
+                                            cb_progress=cb_progress)
+                    new_cr = git.getCurrentCommit(rpath)
+                    if new_cr != cr:
                         info_outdate = True
-                else:
-                    git.resetRepository(rpath, to_rev=revision)
-                    info_outdate = True
             if info_outdate:
+                self.raiseOutdate()
                 dv.Set('cl_update_outdate_set', 'on', force=True)
         finally:
             self.unstash_cache(rpath, name)
         return True
 
+    def raiseOutdate(self):
+        self.clVars.Set('cl_update_outdate_set', 'on', force=True)
+
     def setAutocheckParams(self, status, interval, update_other, cleanpkg):
         """
         Настроить параметры автопроверки обновлений
@@ -247,27 +265,37 @@ class Update(object):
                     self.endTask()
         return True
 
+    @variable_module("update")
     def syncRepositories(self, repname, clean_on_error=True):
         """
         Синхронизировать репозитории
         """
         dv = self.clVars
-        url, rpath, revision, branch = (
+        url, rpath, revision = (
             dv.Select(["cl_update_rep_url", "cl_update_rep_path",
-                       "cl_update_rep_rev", "cl_update_branch_name"],
+                       "cl_update_rep_rev"],
                       where="cl_update_rep_name", eq=repname, limit=1))
         if not url or not rpath:
             raise UpdateError(_("Configuration variables for repositories "
                                 "are not setup"))
+        chroot_path = path.normpath(self.clVars.Get('cl_chroot_path'))
+        # TODO: DEBUG
+        # print "DEBUG", repname, revision
+        if chroot_path == '/':
+            rpath_orig = rpath
+        else:
+            rpath_orig = rpath[len(chroot_path):]
         self.addProgress()
+
         if clean_on_error:
             try:
                 layman = Layman(dv.Get('cl_update_layman_installed'),
                                 dv.Get('cl_update_layman_make'),
-                                dv.Get('cl_update_layman_conf'))
+                                dv.Get('cl_update_layman_conf'),
+                                prefix=chroot_path)
                 if repname != "portage":
-                    layman.add(repname, url, rpath)
-                if not self._syncRepository(repname, url, rpath, revision, branch,
+                    layman.add(repname, url, rpath_orig)
+                if not self._syncRepository(repname, url, rpath, revision,
                                      cb_progress=self.setProgress):
                     return "skip"
                 return True
@@ -279,12 +307,12 @@ class Update(object):
                 self.startTask(
                     _("Re-fetching the {name} repository").format(name=repname))
                 self.addProgress()
+                rpath_new = "%s_new" % rpath
                 try:
-                    rpath_new = "%s_new" % rpath
                     self._syncRepository(repname, url, rpath_new, revision,
-                                         branch, cb_progress=self.setProgress)
+                                         cb_progress=self.setProgress)
                     removeDir(rpath)
-                    os.rename(rpath_new, rpath)
+                    shutil.move(rpath_new, rpath)
                 except OSError as e:
                     raise UpdateError(_("Failed to modify the "
                                         "{repname} repository").format(
@@ -298,9 +326,12 @@ class Update(object):
 
         layman = Layman(dv.Get('cl_update_layman_installed'),
                         dv.Get('cl_update_layman_make'),
-                        dv.Get('cl_update_layman_conf'))
+                        dv.Get('cl_update_layman_conf'),
+                        prefix=chroot_path)
         if repname != "portage":
-            layman.add(repname, url, rpath)
+            # TODO: debug block
+            #print "EEEE",repname, url, rpath_orig
+            layman.add(repname, url, rpath_orig)
         return True
 
     metadata_cache_names = ("metadata/md5-cache", "metadata/cache")
@@ -311,7 +342,7 @@ class Update(object):
         """
         if name in ("portage",):
             return
-        if not name in OverlayOwnCache():
+        if not name in OverlayOwnCache(self.clVars):
             for cachename in self.metadata_cache_names:
                 cachedir = path.join(rpath, cachename)
                 if path.exists(cachedir):
@@ -321,7 +352,7 @@ class Update(object):
                                                    cachename) + ".stash")
                         if path.exists(cachedir_s):
                             removeDir(cachedir_s)
-                        os.rename(cachedir, cachedir_s)
+                        shutil.move(cachedir, cachedir_s)
                     except BaseException as e:
                         pass
 
@@ -332,7 +363,7 @@ class Update(object):
         if name in ("portage",):
             return
         cachenames = self.metadata_cache_names
-        if not name in OverlayOwnCache():
+        if not name in OverlayOwnCache(self.clVars):
             if any(path.exists(path.join(rpath, x)) for x in cachenames):
                 for cachename in cachenames:
                     cachedir_s = path.join(path.dirname(rpath),
@@ -342,7 +373,7 @@ class Update(object):
                             removeDir(cachedir_s)
                         except BaseException as e:
                             pass
-                OverlayOwnCache().add(name)
+                OverlayOwnCache(self.clVars).add(name)
             else:
                 for cachename in cachenames:
                     cachedir = path.join(rpath, cachename)
@@ -350,21 +381,22 @@ class Update(object):
                                            path.basename(cachename)+".stash")
                     if path.exists(cachedir_s):
                         try:
-                            os.rename(cachedir_s, cachedir)
+                            shutil.move(cachedir_s, cachedir)
                         except BaseException as e:
                             pass
         else:
             if all(not path.exists(path.join(rpath, x)) for x in cachenames):
-                OverlayOwnCache().discard(name)
+                OverlayOwnCache(self.clVars).discard(name)
 
 
     def syncLaymanRepository(self, repname):
         """
         Обновить репозиторий через layman
         """
-        layman = getProgPath('/usr/bin/layman')
+        layman = self.get_prog_path('/usr/bin/layman')
         if not layman:
             raise UpdateError(_("The Layman tool is not found"))
+
         rpath = self.clVars.Select('cl_update_other_rep_path',
                                    where='cl_update_other_rep_name', eq=repname,
                                    limit=1)
@@ -386,11 +418,15 @@ class Update(object):
             self.unstash_cache(rpath, laymanname)
         return True
 
+    def _regenCache_process(self, progname, repname, cpu_num):
+        return process(progname, "--repo=%s" % repname, "--update",
+                        "--jobs=%s" % cpu_num, stderr=STDOUT)
+
     def regenCache(self, repname):
         """
         Обновить кэш метаданных репозитория
         """
-        egenCache = getProgPath('/usr/bin/egencache')
+        egenCache = self.get_prog_path('/usr/bin/egencache')
         if not egenCache:
             raise UpdateError(_("The Portage tool is not found"))
         if repname in self.clVars.Get('cl_update_rep_name'):
@@ -407,14 +443,13 @@ class Update(object):
                 raise UpdateError(_("Failed to update the cache of the {rname} "
                                     "repository").format(rname=repname))
         cpu_num = self.clVars.Get('hr_cpu_num')
-        if repname in OverlayOwnCache():
+        if repname in OverlayOwnCache(self.clVars):
             self.printWARNING(
                 _("Repository %s has its own cache") % repname.capitalize())
         else:
             self.startTask(_("Updating the %s repository cache") %
                            repname.capitalize())
-            p = process(egenCache, "--repo=%s" % repname, "--update",
-                        "--jobs=%s" % cpu_num, stderr=STDOUT)
+            p = self._regenCache_process(egenCache, repname, cpu_num)
             if p.failed():
                 raise UpdateError(_("Failed to update the cache of the {rname} "
                                     "repository").format(rname=repname),
@@ -425,7 +460,7 @@ class Update(object):
         """
         Выполнить egencache и emerge --metadata
         """
-        emerge = getProgPath("/usr/bin/emerge")
+        emerge = self.get_prog_path("/usr/bin/emerge")
         if not emerge:
             raise UpdateError(_("The Emerge tool is not found"))
         self.addProgress()
@@ -440,7 +475,10 @@ class Update(object):
             raise UpdateError(_("Failed to update metadata"), addon=data)
         return True
 
-    def eixUpdate(self):
+    def _eixUpdateCommand(self, eix_cmd, countRep):
+        return PercentProgress(eix_cmd, "-F", part=countRep or 1, atty=True)
+
+    def eixUpdate(self, repositroies):
         """
         Выполенине eix-update для репозиторием
 
@@ -448,12 +486,12 @@ class Update(object):
         обновлялись, если cl_update_eixsync_force==auto, либо
         все, если cl_update_eixupdate_force==force
         """
-        eixupdate = getProgPath("/usr/bin/eix-update")
+        eixupdate = self.get_prog_path("/usr/bin/eix-update")
         if not eixupdate:
             raise UpdateError(_("The Eix tool is not found"))
         self.addProgress()
-        countRep = len(self.clVars.Get('main.cl_portdir_overlay'))+1
-        p = PercentProgress(eixupdate, "-F", part=countRep or 1, atty=True)
+        countRep = len(repositroies)
+        p = self._eixUpdateCommand(eixupdate, countRep)
         for perc in p.progress():
             self.setProgress(perc)
         if p.failed():
@@ -552,7 +590,7 @@ class Update(object):
         """
         Запуск команды, которая подразумевает выполнение emerge
         """
-        cmd_path = getProgPath(cmd)
+        cmd_path = self.get_prog_path(cmd)
         if not cmd_path:
             raise UpdateError(_("Failed to find the %s command") % cmd)
         with EmergeParser(
@@ -564,7 +602,7 @@ class Update(object):
         """
         Запуск revdep-rebulid
         """
-        cmd_path = getProgPath(cmd)
+        cmd_path = self.get_prog_path(cmd)
         if not cmd_path:
             raise UpdateError(_("Failed to find the %s command") % cmd)
         with EmergeParser(
@@ -662,7 +700,7 @@ class Update(object):
         Отобразить список удаляемых пакетов
         """
         # подробный список пакетов
-        if self.clVars.Get('cl_update_emergelist_set') == 'on':
+        if self.clVars.Get('update.cl_update_emergelist_set') == 'on':
             self.printPre(self._emerge_translate(
                 emerge.uninstall_packages.verbose_result))
         else:
@@ -677,9 +715,7 @@ class Update(object):
         Получить список обновляемых пакетов @world из кэша
         """
         if "@world" in packages:
-            from calculate.update.utils.cl_update import ClUpdateAction
-            elog = EmergeLog(
-                EmergeLogNamedTask(ClUpdateAction.log_names['premerge']))
+            elog = EmergeLog(EmergeLogNamedTask(EmergeMark.Premerge))
             if check and (elog.list or elog.remove_list):
                 self.emerge_cache.drop_cache(
                     "Some packages was installed or removed")
@@ -697,9 +733,7 @@ class Update(object):
         premerge
         """
         self.emerge_cache.set_cache(pkg_list)
-        from calculate.update.utils.cl_update import ClUpdateAction
-        elog = EmergeLog(
-            EmergeLogNamedTask(ClUpdateAction.log_names['premerge']))
+        elog = EmergeLog(EmergeLogNamedTask(EmergeMark.Premerge))
         elog.mark_end_task(),
 
     def mark_schedule(self):
@@ -708,6 +742,9 @@ class Update(object):
         """
         SystemIni().setVar('update', {'last_check': str(int(time.time()))})
 
+    def get_default_emerge_opts(self):
+        return self.clVars.Get('cl_emerge_default_opts')
+
     def premerge(self, param, *packages):
         """
         Вывести информацию об обновлении
@@ -739,6 +776,8 @@ class Update(object):
                     return True
                 emergelike = self.clVars.Get('cl_update_emergelist_set') == 'on'
                 self._display_install_package(emerge, emergelike)
+                if str(emerge.skipped_packages):
+                    self._display_error(emerge.skipped_packages)
             except EmergeError:
                 self.set_need_update(False)
                 self.emerge_cache.drop_cache("Emerge error")
@@ -761,7 +800,7 @@ class Update(object):
         """
         Установить флаг: есть обновления
         """
-        if self.clVars.Get('cl_update_autocheck_set') == 'off':
+        if self.clVars.Get('update.cl_update_autocheck_set') == 'off':
             val = False
         UpdateInfo.set_update_ready(val)
         return True
@@ -812,10 +851,15 @@ class Update(object):
         error = "<br/>".join(str(error).split('<br/>')[-lines_num:])
         self.printPre(self._emerge_translate(error))
 
-    def emerge(self, param, *packages):
+    def emerge(self, use, param, *packages):
         """
         Выполнить сборку пакета
         """
+        if self.clVars.Get('cl_chroot_path') != '/':
+            command_class = partial(ChrootEmergeCommand,
+                self.clVars.Get('cl_chroot_path'))
+        else:
+            command_class = EmergeCommand
         deo = self.clVars.Get('cl_emerge_default_opts')
         if not packages:
             packages = [param]
@@ -825,8 +869,9 @@ class Update(object):
             if not packages:
                 return True
             extra_params = [param]
-        with EmergeParser(EmergeCommand(list(packages), emerge_default_opts=deo,
-                                        extra_params=extra_params)) as emerge:
+        with EmergeParser(command_class(list(packages), emerge_default_opts=deo,
+                                        extra_params=extra_params,
+                                        use=use)) as emerge:
             try:
                 emerge.question.action = lambda x: False
                 emerge.run()
@@ -843,7 +888,7 @@ class Update(object):
         """
         Выполнить очистку системы от лишних пакетов
         """
-        deo = self.clVars.Get('cl_emerge_default_opts')
+        deo = self.get_default_emerge_opts()
         emerge = None
         try:
             emerge = EmergeParser(EmergeCommand(["--depclean"],
@@ -911,79 +956,95 @@ class Update(object):
 
         return decor
 
-    def migrateCacheRepository(self, url, branch):
-        rep_set = self.clVars.Get('cl_update_profile_storage')
-        rep = rep_set.get_repository(url, branch)
+    def migrateCacheRepository(self, url, branch, storage):
+        """
+        Перенести репозиторий из кэша в локальный
+        """
+        rep = storage.get_repository(url, branch)
         if rep:
-            rep.storage = rep_set.storages[0]
+            rep.storage = storage.storages[0]
             self.clVars.Invalidate('cl_update_profile_storage')
         return True
 
 
-    def reconfigureProfileVars(self):
+    def reconfigureProfileVars(self, profile_dv, chroot):
         """
         Синхронизировать репозитории
         """
         dv = self.clVars
 
-        profile_dv = dv.Get('cl_update_profile_datavars')
         try:
             if not profile_dv:
                 raise UpdateError(
                     _("Failed to use the new profile. Try again."))
-            for var_name in ('cl_update_rep_rev',
+            for var_name in (#'cl_update_rep_rev',
                              'cl_update_rep_path',
                              'cl_update_rep_url',
                              'cl_update_rep_name',
+                             'cl_update_branch',
+                             'cl_update_binhost_list',
                              'cl_update_branch_name',
                              'cl_profile_system',
-                             'cl_update_layman_storage',
-                             'cl_update_rep'):
+                             'cl_update_rep'
+                             ):
+                # TODO: debug block
+                #print var_name, ":", profile_dv.Get(var_name)
                 dv.Set(var_name, profile_dv.Get(var_name), force=True)
-        except DataVarsError:
+            dv.Set('cl_chroot_path', chroot, force=True)
+            # TODO: debug block
+            #print ('cl_builder_branch_name',
+            #       self.clVars.Get('cl_builder_branch_name'))
+        except DataVarsError as e:
+            print str(e)
             raise UpdateError(_("Wrong profile"))
         return True
 
-    def setProfile(self):
-        profile = self.clVars.Select('cl_profile_path',
-            where='cl_profile_shortname',
-            eq=self.clVars.Get('cl_update_profile_system'), limit=1)
+    def setProfile(self, profile_shortname):
+        profile = self.clVars.Select('cl_update_profile_path',
+            where='cl_update_profile_shortname',
+            eq=profile_shortname, limit=1)
         if not profile:
             raise UpdateError(_("Failed to determine profile %s") %
                               self.clVars.Get('cl_update_profile_system'))
         profile_path = path.relpath(profile, '/etc/portage')
         try:
-            for rm_fn in filter(path.exists,
+            profile_file = '/etc/portage/make.profile'
+            if not path.exists(
+                    path.join(path.dirname(profile_file), profile_path)):
+                raise UpdateError(
+                    _("Failed to set the profile: %s")%_("Profile not found"))
+            for rm_fn in filter(path.lexists,
                                 ('/etc/make.profile', '/etc/portage/make.profile')):
                 os.unlink(rm_fn)
-            os.symlink(profile_path, '/etc/portage/make.profile')
-        except (OSError,IOError) as e:
+            os.symlink(profile_path, profile_file)
+        except (OSError, IOError) as e:
             raise UpdateError(_("Failed to set the profile: %s")%str(e))
         return True
 
-    def applyProfileTemplates(self,useClt=None,cltFilter=False,useDispatch=True):
+    def applyProfileTemplates(self, useClt=None, cltFilter=False,
+                              useDispatch=True, action="merge"):
         """
         Наложить шаблоны из профиля
         """
-        from calculate.lib.cl_template import (Template,TemplatesError,
-            ProgressTemplate)
+        from calculate.lib.cl_template import TemplatesError, ProgressTemplate
         dv = DataVarsUpdate()
         try:
             dv.importUpdate()
             dv.flIniFile()
-            dv.Set('cl_action','merge',force=True)
-            dv.Set('cl_templates_locate', self.clVars.Get('cl_update_templates_locate'))
-            dv.Set("cl_chroot_path",'/', True)
-            dv.Set("cl_root_path",'/', True)
-            for copyvar in ("cl_dispatch_conf", "cl_verbose_set"):
-                dv.Set(copyvar,self.clVars.Get(copyvar),True)
+            dv.Set('cl_action', action, force=True)
+            dv.Set('cl_templates_locate',
+                   self.clVars.Get('cl_update_templates_locate'))
+            dv.Set("cl_chroot_path", '/', True)
+            dv.Set("cl_root_path", '/', True)
+            for copyvar in ("cl_dispatch_conf", "cl_verbose_set",
+                            "update.cl_update_world"):
+                dv.Set(copyvar, self.clVars.Get(copyvar), True)
             # определение каталогов содержащих шаблоны
-            useClt = useClt in ("on",True)
+            useClt = useClt in ("on", True)
             self.addProgress()
-            nullProgress = lambda *args,**kw:None
+            nullProgress = lambda *args, **kw: None
             dispatch = self.dispatchConf if useDispatch else None
-            clTempl = ProgressTemplate(nullProgress, dv,
-                                       cltObj=useClt,
+            clTempl = ProgressTemplate(nullProgress, dv, cltObj=useClt,
                                        cltFilter=cltFilter,
                                        printSUCCESS=self.printSUCCESS,
                                        printWARNING=self.printWARNING,
@@ -1006,7 +1067,7 @@ class Update(object):
 
     def cleanpkg(self):
         """
-        Очистить distfiles и pkgdir от устаревших пакетов
+        Очистить PKGDIR и DISTFILES в текущей системе
         """
         portdirs = ([self.clVars.Get('cl_portdir')] +
                     self.clVars.Get('cl_portdir_overlay'))
@@ -1015,6 +1076,20 @@ class Update(object):
         distdir = self.clVars.Get('install.cl_distfiles_path')
         pkgdir = self.clVars.Get('cl_pkgdir')
 
+        logger = log("update_cleanpkg.log",
+                     filename="/var/log/calculate/update_cleanpkg.log",
+                     formatter="%(asctime)s - %(clean)s - %(message)s")
+
+        return self._cleanpkg(
+            distdir, pkgdir, distdirfiles, pkgfiles, logger)
+
+    def _update_binhost_packages(self):
+        os.system('/usr/sbin/emaint binhost -f &>/dev/null')
+
+    def _cleanpkg(self, distdir, pkgdir, distdirfiles, pkgfiles, logger):
+        """
+        Общий алгоритм очистки distfiles и pkgdir от устаревших пакетов
+        """
         skip_files = ["/metadata.dtd", "/Packages"]
         try:
             if self.clVars.Get('client.os_remote_auth'):
@@ -1022,14 +1097,11 @@ class Update(object):
         except DataVarsError:
             pass
 
-        logger = log("update_cleanpkg.log",
-                     filename="/var/log/calculate/update_cleanpkg.log",
-                     formatter="%(asctime)s - %(clean)s - %(message)s")
-
-        for cleantype, filelist in (("packages",
-                    get_remove_list(pkgdir, list(pkgfiles), depth=4)),
-                   ("distfiles",
-                    get_remove_list(distdir, list(distdirfiles), depth=1))):
+        for cleantype, filelist in (
+                ("packages",
+                 get_remove_list(pkgdir, list(pkgfiles), depth=4)),
+                ("distfiles",
+                 get_remove_list(distdir, list(distdirfiles), depth=1))):
             removelist = []
             for fn in filelist:
                 try:
@@ -1043,7 +1115,7 @@ class Update(object):
                 logger.info(removelist_str, extra={'clean': cleantype})
                 if cleantype == "packages":
                     try:
-                        os.system('/usr/sbin/emaint binhost -f &>/dev/null')
+                        self._update_binhost_packages()
                         for dn in listDirectory(pkgdir, fullPath=True):
                             if path.isdir(dn) and not listDirectory(dn):
                                 os.rmdir(dn)
@@ -1055,3 +1127,60 @@ class Update(object):
         cache = SetupCache(self.clVars)
         cache.update(force=True)
         return True
+
+    def check_binhost(self, write_binhost=True):
+        """
+        Проверить, что доступен хотя бы один из binhost'ов
+        :return:
+        """
+        hosts =self.clVars.Get("update.cl_update_binhost_host")
+        if not hosts:
+            self.clVars.Delete('cl_update_binhost', location="system")
+            raise UpdateError("Binhost is unavailable")
+        if write_binhost:
+            if hosts[0] != self.clVars.Get('update.cl_update_binhost'):
+                self.refresh_binhost = True
+            self.clVars.Write('cl_update_binhost', hosts[0], location="system")
+        return True
+
+    def message_binhost_changed(self):
+        if self.refresh_binhost:
+            self.printWARNING(_("Update server was changed to %s") %
+                              self.clVars.Get('cl_update_binhost'))
+        return True
+
+    def update_binhost_list(self):
+        """
+        Обновить список binhost'ов после обновления до master веток
+        :return:
+        """
+        dv = DataVarsUpdate()
+        try:
+            dv.importUpdate()
+            dv.flIniFile()
+            changes = False
+            for varname in ('update.cl_update_binhost_list',
+                            'update.cl_update_binhost_timestamp_path',
+                            'cl_update_binhost_revision_path'):
+                new_value = dv.Get(varname)
+                old_value = self.clVars.Get(varname)
+                if new_value != old_value:
+                    changes = True
+                self.clVars.Set(varname, new_value, force=True)
+            if not changes:
+                raise UpdateError("Binhost is unavailable")
+        except DataVarsError:
+            raise UpdateError("Binhost is unavailable")
+        return True
+
+    def drop_binhosts(self, dv):
+        """
+        Обновление до master веток
+        """
+        branch = dv.Get('update.cl_update_branch')
+        revs = [
+            branch for x in dv.Get('update.cl_update_rep_name')
+        ]
+        dv.Set('update.cl_update_branch_name', revs)
+        dv.Invalidate('update.cl_update_rep_rev')
+        return True
